package com.persagy.dmp.report.common.json;

import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.*;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.JsonNodeType;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.gson.reflect.TypeToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class JsonMapper {

    private static Logger logger = LoggerFactory.getLogger(JsonMapper.class);

    private static final ThreadLocal<JsonMapper> emptyMapper = new ThreadLocal<JsonMapper>();
    private static final ThreadLocal<JsonMapper> defaultMapper = new ThreadLocal<JsonMapper>();

    public static boolean isEmpty(final CharSequence cs) {
        return cs == null || cs.length() == 0;
    }

    private ObjectMapper mapper;

    @SuppressWarnings("unused")
    private JsonMapper() {
        this(null);
    }

    public JsonMapper(Include include) {
        mapper = new ObjectMapper();
        if (include != null) {
            mapper.setSerializationInclusion(include);
        }
        mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
    }

    public static ObjectMapper nonEmptyObjectMapper() {
        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.setSerializationInclusion(Include.NON_EMPTY);
        objectMapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
        objectMapper.disable(DeserializationFeature.FAIL_ON_IGNORED_PROPERTIES);
        return objectMapper;
    }

    public static JsonMapper nonEmptyMapper() {
        JsonMapper jm = emptyMapper.get();
        if (jm == null) {
            jm = new JsonMapper(Include.NON_EMPTY);
            emptyMapper.set(jm);
        }
        return jm;
    }

    public static JsonMapper nonDefaultMapper() {
        JsonMapper jm = defaultMapper.get();
        if (jm == null) {
            jm = new JsonMapper(Include.NON_DEFAULT);
            defaultMapper.set(jm);
        }
        return jm;
    }

    /**
     * 简单调用
     *
     * @param object
     * @return
     */
    public static String toSimpleJson(Object object) {
        return JsonMapper.nonEmptyMapper().toJson(object);
    }

    public static <T> T fromJsonSimple(String jsonString, Class<T> clazz) {
        return JsonMapper.nonDefaultMapper().fromJson(jsonString, clazz);
    }

    /**
     * @param object
     * @return
     */
    public String toJson(Object object) {

        try {
            return mapper.writeValueAsString(object);
        } catch (IOException e) {
            logger.warn("write to json string error:" + object, e);
            return null;
        }
    }

    /**
     * @param jsonString
     * @param clazz
     * @return
     */
    public <T> T fromJson(String jsonString, Class<T> clazz) {
        if (isEmpty(jsonString)) {
            return null;
        }

        try {
            return mapper.readValue(jsonString, clazz);
        } catch (IOException e) {
            logger.warn("parse json string error:" + jsonString, e);
            return null;
        }
    }

    /**
     * @param jsonString
     * @param javaType
     * @return
     */
    @SuppressWarnings("unchecked")
    public <T> T fromJson(String jsonString, JavaType javaType) {
        if (isEmpty(jsonString)) {
            return null;
        }

        try {
            return (T) mapper.readValue(jsonString, javaType);
        } catch (IOException e) {
            logger.warn("parse json string error:" + jsonString, e);
            return null;
        }
    }

    /**
     * 构造Collection类型.
     */
    @SuppressWarnings("rawtypes")
    public JavaType contructCollectionType(Class<? extends Collection> collectionClass, Class<?> elementClass) {
        return mapper.getTypeFactory().constructCollectionType(collectionClass, elementClass);
    }

    /**
     * 构造Map类型.
     */
    @SuppressWarnings("rawtypes")
    public JavaType contructMapType(Class<? extends Map> mapClass, Class<?> keyClass, Class<?> valueClass) {
        return mapper.getTypeFactory().constructMapType(mapClass, keyClass, valueClass);
    }

    /**
     * 取出Mapper.
     */
    public static ObjectMapper getMapper() {
        ObjectMapper mapper = new ObjectMapper();
        mapper.setSerializationInclusion(Include.NON_EMPTY);
        mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
        return mapper;
    }

    public static String toFormatJson(Object object) {
        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.setSerializationInclusion(Include.NON_EMPTY);
        objectMapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
        objectMapper.enable(SerializationFeature.INDENT_OUTPUT);

        try {
            return objectMapper.writeValueAsString(object);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static Map<String, Object> json2map(String str_json) {
        Map<String, Object> res = null;
        try {
            Gson gson = new Gson();
            res = gson.fromJson(str_json, new TypeToken<Map<String, Object>>() {
            }.getType());
        } catch (JsonSyntaxException e) {
        }
        return res;
    }

    public static List<Object> json2List(String str_json) {
        List<Object> res = null;
        try {
            Gson gson = new Gson();
            res = gson.fromJson(str_json, new TypeToken<List<Object>>() {
            }.getType());
        } catch (JsonSyntaxException e) {
        }
        return res;
    }

    public static Integer getInt(ObjectNode obj, String key) {
        if (obj == null) {
            return null;
        }
        JsonNode node = obj.get(key);
        if (node == null) {
            return null;
        }
        JsonNodeType nodeType = node.getNodeType();
        if (node.isInt()) {
            return node.asInt();
        }
        if (nodeType == JsonNodeType.NUMBER) {
            return node.intValue();
        } else {
            String str = node.asText();
            return Double.valueOf(str).intValue();
        }
    }

    public static Integer getInt(ObjectNode obj, String key, Integer defaultValue) {
        Integer result = getInt(obj, key);
        return result == null ? defaultValue : result;

    }

    public static Double getDouble(ObjectNode obj, String key) {
        if (obj == null) {
            return null;
        }
        JsonNode node = obj.get(key);
        if (node == null) {
            return null;
        }
        JsonNodeType nodeType = node.getNodeType();
        if (node.isDouble()) {
            return node.asDouble();
        }
        if (nodeType == JsonNodeType.NUMBER) {
            return node.doubleValue();
        } else {
            String str = node.asText();
            return Double.valueOf(str);
        }
    }

    public static Double getDouble(ObjectNode obj, String key, Double defaultValue) {
        Double result = getDouble(obj, key);
        return result == null ? defaultValue : result;
    }

    public static String getString(ObjectNode obj, String key) {
        if (obj == null) {
            return null;
        }
        JsonNode node = obj.get(key);
        if (node == null) {
            return null;
        }
        if (node.getNodeType() == JsonNodeType.STRING) {
            return node.asText();
        }
        return node.toString();
    }

    public static String getString(ObjectNode obj, String key, String defaultValue) {
        String result = getString(obj, key);
        return result == null ? defaultValue : result;
    }

    public static ObjectNode getObject(ObjectNode obj, String key, String defaultValue) {
        ObjectNode result = getObject(obj, key);
        return result == null ? JsonMapper.fromJsonSimple(defaultValue, ObjectNode.class) : result;
    }

    public static ObjectNode getObject(ObjectNode obj, String key) {
        if (obj == null) {
            return null;
        }
        JsonNode node = obj.get(key);
        if (node == null) {
            return null;
        }
        JsonNodeType nodeType = node.getNodeType();
        if (nodeType == JsonNodeType.OBJECT && node.isObject()) {
            return (ObjectNode) node;
        }
        return null;
    }

    public static ArrayNode getArray(ObjectNode obj, String key) {
        if (obj == null) {
            return null;
        }
        JsonNode node = obj.get(key);
        if (node == null) {
            return null;
        }
        JsonNodeType nodeType = node.getNodeType();
        if (nodeType == JsonNodeType.ARRAY && node.isArray()) {
            return (ArrayNode) node;
        }
        return JsonNodeFactory.instance.arrayNode();
    }
}
